(********************************************************************
 * COPYRIGHT --  
 ********************************************************************
 * Package: AxisAutomat
 * File: AxisAutomat.typ
 * Author: chenh
 * Created: January 13, 2016
 ********************************************************************
 * Data types of package AxisAutomat
 ********************************************************************)
(*Global axis command and monitor type*)

TYPE
	global_axes_monitor_type : 	STRUCT  (*global axes parameter monitor*)
		ActPosition : REAL; (*actual position of the axis*)
		ActVelocity : REAL; (*actual velocity of the axis*)
		ActCurrent : REAL; (*ActCurrent*)
		ActTorque : REAL; (*ActTorque*)
		MaxCurrent : REAL; (*MaxCurrent*)
		ActDCVoltage : REAL; (*ActDCVoltage*)
		ActMotorTemperature : REAL; (*ActMotorTemperature*)
		ActACLine : REAL; (*ActACLine*)
		ActRpm : REAL; (*ActRpm*)
	END_STRUCT;
END_TYPE

(*used for write single ID*)

TYPE
	AxisParaType_Enum : 
		(
		TYPE_DINT := 4,
		TYPE_USINT := 5,
		TYPE_UINT := 6,
		TYPE_UDINT := 7,
		TYPE_REAL := 8
		);
	AxisParaID_typ : 	STRUCT 
		paraID : UINT;
		datatype : UINT;
		DataAddress : UDINT;
		UI1 : USINT; (*value*)
		UI2 : UINT; (*value*)
		UI4 : UDINT; (*value*)
		I4 : DINT; (*value*)
		R4 : REAL; (*value*)
	END_STRUCT;
END_TYPE

(*this struct is used for triggstop functon*)

TYPE
	axis_triggstop_pra_type : 	STRUCT 
		Velocity : REAL;
		Acceleration : REAL;
		Deceleration : REAL;
		TriggerDistance : REAL;
		TriggerInput : MC_TRIGGER_REF;
		Direction : USINT;
	END_STRUCT;
END_TYPE

(*Real axis control struct type*)

TYPE
	axis_command_type : 	STRUCT  (*command structure for  axis*)
		Power : BOOL; (*switch on the controller*)
		PowerOff : BOOL; (*switch off the controller*)
		Home : BOOL; (*reference axis*)
		MoveJogPos : BOOL; (*move in positive direction as long as is set*)
		MoveJogNeg : BOOL; (*move in negative direction as long as is set*)
		MoveAbsolute : BOOL; (*move to a defined position*)
		MoveVelocityTtiggStop : BOOL; (*move with a defined speed*)
		MoveVelocity : BOOL; (*move with a defined speed*)
		MoveAdditive : BOOL; (*move a defined distance*)
		Stop : BOOL; (*stop every active movement on the axis as long as is set*)
		Halt : BOOL; (*stop every active movement on the axis*)
		AutCamInit : BOOL; (*start coupling beetween master and slave*)
		AutStartSlave : BOOL; (*start coupling beetween master and slave*)
		AutDisengageSlave : BOOL; (*stop coupling between master and slave*)
		Signal2 : BOOL;
		Signal3 : BOOL;
		Signal4 : BOOL;
		Signal1 : BOOL;
		CamStart : BOOL;
		CamStop : BOOL;
		resetGlobalInit : BOOL;
		WriteGrpID : BOOL;
		WriteSingleID : BOOL;
		ErrorAcknowledge : BOOL;
	END_STRUCT;
	axis_encoder_parameter_type : 	STRUCT 
		count_dir : USINT;
		units : UDINT;
		rev_motor : UDINT;
	END_STRUCT;
	axis_control_parameter_type : 	STRUCT 
		Kpp : REAL;
		Kvp : REAL;
		Tn_p : REAL;
		T_filter : REAL;
		T_predict : REAL;
		T_total : REAL;
	END_STRUCT;
	cam_parameter_typ : 	STRUCT 
		MasterOffset : REAL; (*offset of master*)
		SlaveOffset : REAL; (*offset of slave*)
		MasterScaling : REAL; (*scaling of master axis *)
		SlaveScaling : REAL; (*scaling of slave axis*)
		StartMode : USINT; (*start mode*)
	END_STRUCT;
	axis_limit_parameter_type : 	STRUCT 
		a1_pos : REAL;
		a1_neg : REAL;
		a2_pos : REAL;
		a2_neg : REAL;
		a_stop : REAL;
		ds_stop : REAL;
		v_pos : REAL;
		v_neg : REAL;
	END_STRUCT;
	axis_parameter_type : 	STRUCT  (*parameter structure*)
		Position : REAL; (*target position for MoveAbsolute-Command*)
		Distance : REAL; (*distance for MoveAdditive-Command*)
		Velocity : REAL; (*velocity for MoveVelocity-Command*)
		JogVelocity : REAL; (*velocity for jogging movement*)
		Direction : USINT; (*direction for commanded movements*)
		Acceleration : REAL; (*acceleration for commanded movements*)
		Deceleration : REAL; (*deceleration for commanded movements*)
		HomePosition : REAL; (*target position for referencing the axis*)
		HomeMode : USINT; (*homing mode*)
		V_Override : REAL;
		A_Override : REAL;
		V_Max : REAL;
		V_Home : REAL;
		TriggerStopPra : axis_triggstop_pra_type;
		camProfileName : ARRAY[0..MAX_AXIS_DB_NUM_MINUS_ONE]OF STRING[12];
		camProfileIndex : ARRAY[0..MAX_AXIS_DB_NUM_MINUS_ONE]OF USINT;
		paraTableName : ARRAY[0..MAX_AXIS_DB_NUM_MINUS_ONE]OF STRING[12];
		WriteGrpID : ARRAY[0..MAX_AXIS_PARID_NUM_MINUS_ONE]OF AxisParaID_typ;
		WriteSingleID : AxisParaID_typ;
		LimPara : axis_limit_parameter_type;
		EncoderPara : axis_encoder_parameter_type;
		Control : axis_control_parameter_type;
		CamPara : cam_parameter_typ;
	END_STRUCT;
	axis_status_type : 	STRUCT  (*status structure*)
		ActPosition : REAL; (*actual position of the axis*)
		ActVelocity : REAL; (*actual velocity of the axis*)
		ActCurrent : REAL;
		ActTorque : REAL;
		MaxCurrent : REAL;
		ActDCVoltage : REAL;
		ActMotorTemperature : REAL;
		ActACLine : REAL;
		ActRpm : REAL;
		DriveStatus : MC_DRIVESTATUS_TYP; (*actual status of the axis*)
		RefOk : BOOL;
		TriggStopOk : BOOL;
		ActualStateIndex : USINT; (*Automat states index*)
		HomeEnable : BOOL;
		camProfDownInitOk : BOOL;
		paraTabDownInitOk : BOOL;
		AutCamInitOk : BOOL;
		globalInit : BOOL;
		WriteGrpID_Ok : BOOL;
		WriteSingleID_Ok : BOOL;
	END_STRUCT;
	axis_state_type : 	STRUCT  (*axis state structure*)
		Disabled : BOOL; (*if set, axis is in state Disabled*)
		StandStill : BOOL; (*if set, axis is in state StandsStill*)
		Homing : BOOL; (*if set, axis is in state Homing*)
		Stopping : BOOL; (*if set, axis is in state Stopping*)
		DiscreteMotion : BOOL; (*if set, axis is in state DiscreteMotion*)
		ContinuousMotion : BOOL; (*if set, axis is in state ContinuousMotion*)
		SynchronizedMotion : BOOL; (*if set, axis is in state SynchronizedMotion*)
		ErrorStop : BOOL; (*if set, axis is in state ErrorStop*)
	END_STRUCT;
	axis_error_type : 	STRUCT  (*error structure *)
		ErrorText : ARRAY[0..3]OF STRING[79]; (*description of first active error or warning*)
		ErrorID : UINT; (*ErrorID of any occured error*)
	END_STRUCT;
	axis_type : 	STRUCT  (*substructure for single and master axes*)
		Command : axis_command_type; (*command structure for single and master axes*)
		Parameter : axis_parameter_type; (*parameter structure*)
		Status : axis_status_type; (*status structure*)
		AxisState : axis_state_type; (*axis state structure*)
		Error : axis_error_type; (*error structure*)
	END_STRUCT;
END_TYPE

(*axis control machine state*)

TYPE
	STATE_type : 
		( (*State variables for axis state machine*)
		STATE_WAIT := 0,
		STATE_NET_INIT,
		STATE_DOWNLOAD_CAM_PROFILE,
		STATE_WAIT_DOWNLOAD_CAM_PROFILE,
		STATE_DOWNLOAD_PARA_TABLE,
		STATE_WAIT_DOWNLOAD_PARA_TABLE,
		STATE_CHECK_POWER,
		STATE_GLOBAL_INIT,
		STATE_WAIT_GLOBAL_INIT,
		STATE_INIT_POWER_ON,
		STATE_HOME_DIRECT,
		STATE_INIT_STOP,
		STATE_WAIT_INIT_STOP := 100,
		STATE_INIT_POWER_OFF,
		STATE_WAIT_INIT_POWER_OFF,
		STATE_POWER_ON,
		STATE_POWER_OFF,
		STATE_READY,
		STATE_HOME,
		STATE_HALT,
		STATE_STOP,
		STATE_JOG_NEGATIVE,
		STATE_JOG_POSITIVE,
		STATE_MOVE_VELOCITY := 200,
		STATE_MOVE_VELOCITY_TRIGGSTOP,
		STATE_MOVE_ADDITIVE,
		STATE_MOVE_ABSOLUTE,
		STATE_AUTOMAT_INIT,
		STATE_AUTOMAT_START,
		STATE_AUTOMAT_STOP,
		STATE_CAM_SELECT,
		STATE_CAM_START,
		STATE_CAM_STOP,
		STATE_WRITE_PARID := 300,
		STATE_ERROR := 310,
		STATE_ERROR_STOP := 320,
		STATE_ERROR_AXIS := 330,
		STATE_ERROR_RESET := 340
		);
END_TYPE
